const childProcess = require("child_process");
const {
  showInfoText,
  showSuccessText,
  showErrorText,
  showBigSuccessText,
  showLogText,
} = require("../utils");

const localBuild = async function (config) {
  try {
    const { script } = config;
    showInfoText("本地打包中...");
    await new Promise((resolve, reject) => {
      childProcess.exec(
        script,
        { cwd: process.cwd(), maxBuffer: 5000 * 1024 },
        (e) => {
          if (e) {
            reject(e);
          } else {
            resolve();
          }
        }
      );
    });
  } catch (e) {
    showErrorText("打包失败：" + e);
    process.exit(1);
  }
};
const fs = require("fs");
const archiver = require("archiver");

// 压缩本地文件
const createZip = async (config) => {
  try {
    const { localPath } = config;
    showInfoText("压缩中...");
    await new Promise((resolve, reject) => {
      //  创建最终打包文件的输出流
      const output = fs.createWriteStream(`${process.cwd()}/${localPath}.zip`);
      // 生成archiver对象，打包类型为zip
      const archive = archiver("zip", {
        zlib: { level: 9 }, // Sets the compression level.
      });
      output.on("close", (e) => {
        if (e) {
          showErrorText("压缩失败：" + e);
          reject(e);
          process.exit(1);
        } else {
          showSuccessText("打包成功");
          resolve();
        }
      });
      // 将打包对象与输出流关联
      archive.pipe(output);
      // 从子目录追加文件，将其内容放在archive的根目录
      archive.directory(config.localPath, false);
      // 结束archive
      archive.finalize();
    });
  } catch (e) {
    showErrorText("压缩失败：" + e);
    process.exit(1);
  }
};

const { NodeSSH } = require("node-ssh");
const ssh = new NodeSSH();

const connactSSH = async (config) => {
  try {
    showInfoText("服务器连接中...");
    await ssh.connect({
      host: config.host,
      username: config.username,
      password: config.password,
      port: config.port,
    });
    showSuccessText("服务器连接成功");
  } catch (e) {
    showErrorText("服务器连接失败：" + e);
    process.exit(1);
  }
};

const uploadFile = async (config) => {
  try {
    showInfoText("本地打包文件上传中...");
    const { localPath, remotePath } = config;
    await ssh.putFile(
      `${process.cwd()}/${localPath}.zip`,
      `${remotePath}build.zip`
    );
    showSuccessText("本地打包文件上传成功");
  } catch (e) {
    showErrorText("本地打包文件上传失败：" + e);
    process.exit(1);
  }
};

const unzipFile = async (config) => {
  try {
    showInfoText("远程文件解压中...");
    const { remotePath, needEmail } = config;
    // "remotePath": "/root/www/",
    const remoteFileName = `${remotePath}build.zip`;
    await ssh.execCommand(
      "cd " + remotePath + " && rm -rf `ls | grep -v build.zip`"
    );
    await ssh.execCommand(
      `unzip -o ${remoteFileName} -d ${remotePath} && rm -rf ${remoteFileName}`
    );
    // 解压成功后如果有发邮件的需求则发送邮件通知
    if (needEmail) {
      await ssh.execCommand(
        `echo '${config.content}' | mail -s '${config.title}' ${config.addressee}`
      );
    }
    showSuccessText("远程文件解压成功");
  } catch (e) {
    showErrorText("远程文件解压失败：" + e);
    process.exit(1);
  }
};

const deleteLocalFile = (config) => {
  try {
    showInfoText("删除本地打包文件中...");
    let { localPath } = config;

    // function deleteFn(path) {
    //     if (fs.existsSync(path)) {
    //     	// 遍历文件夹，如果里面是文件就直接删掉，是文件夹则递归进行删除
    //         fs.readdirSync(path).forEach((file) => {
    //             var curPath = path + "/" + file;
    //             if (fs.statSync(curPath).isDirectory()) {
    //                 deleteFn(curPath);
    //             } else {
    //                 fs.unlinkSync(curPath);
    //             }
    //         });
    //         fs.rmdirSync(path);
    //     }
    // }
    // deleteFn(localPath)
    fs.unlinkSync(`${localPath}.zip`);
    showSuccessText("删除本地打包文件成功");
  } catch (e) {
    showErrorText("删除本地打包文件失败：" + e);
    process.exit(1);
  }
};

const disconnectSSH = () => {
  ssh.dispose();
};

const path = require("path");
const deployConfigPath = `${path.join(process.cwd())}/deploy.config.js`;

module.exports = {
  description: "构建部署命令",
  option: "-t, --type <type>",
  optionDescription: "setup deploy type",
  run: async function (type) {
    const config = require(deployConfigPath)[type];
    if (!type) {
      showErrorText("请选择要构建的平台");
      process.exit(1);
    } else if (!config) {
      showErrorText(`暂无${type}平台的配置信息，请进行配置`);
      process.exit(1);
    } else {
      if (fs.existsSync(deployConfigPath)) {
        await localBuild(config);
        await createZip(config);
        await connactSSH(config);
        await uploadFile(config);
        await unzipFile(config);
        await deleteLocalFile(config);
        disconnectSSH();
        showBigSuccessText('部署成功');
        showLogText(`总耗时：${Math.floor(process.uptime())}s`)
        process.exit(0);
      } else {
        showErrorText(
          "deploy.config.js配置文件不存在，请先执行auto-deploy init命令创建"
        );
        process.exit(1);
      }
    }
  },
};
